Package: distinct
Type: Package
Title: distinct: a method for differential analyses via hierarchical
        permutation tests
Version: 1.22.0
Author: Simone Tiberi [aut, cre].
Maintainer: Simone Tiberi <simone.tiberi@uzh.ch>
Description: distinct is a statistical method to perform differential testing between two or more groups of distributions; 
  differential testing is performed via hierarchical non-parametric permutation tests on the cumulative distribution functions (cdfs) of each sample.
  While most methods for differential expression target differences in the mean abundance between conditions, 
  distinct, by comparing full cdfs, identifies, both, differential patterns involving changes in the mean,
  as well as more subtle variations that do not involve the mean (e.g., unimodal vs. bi-modal distributions with the same mean).
  distinct is a general and flexible tool: due to its fully non-parametric nature, 
  which makes no assumptions on how the data was generated, it can be applied to a variety of datasets.
  It is particularly suitable to perform differential state analyses on single cell data (i.e., differential analyses within sub-populations of cells), 
  such as single cell RNA sequencing (scRNA-seq) and high-dimensional flow or mass cytometry (HDCyto) data.
  To use distinct one needs data from two or more groups of samples (i.e., experimental conditions), with at least 2 samples (i.e., biological replicates) per group.
biocViews: Genetics, RNASeq, Sequencing, DifferentialExpression,
        GeneExpression, MultipleComparison, Software, Transcription,
        StatisticalMethod, Visualization, SingleCell, FlowCytometry,
        GeneTarget
License: GPL (>= 3)
Depends: R (>= 4.3)
Imports: Rcpp, stats, SummarizedExperiment, SingleCellExperiment,
        methods, Matrix, foreach, parallel, doParallel, doRNG, ggplot2,
        limma, scater
Suggests: knitr, rmarkdown, testthat, UpSetR, BiocStyle
LinkingTo: Rcpp, RcppArmadillo
SystemRequirements: C++17
VignetteBuilder: knitr
RoxygenNote: 7.2.3
ByteCompile: true
URL: https://github.com/SimoneTiberi/distinct
BugReports: https://github.com/SimoneTiberi/distinct/issues
git_url: https://git.bioconductor.org/packages/distinct
git_branch: RELEASE_3_22
git_last_commit: 8dca3b7
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 03:42:07 UTC; biocbuild
