#' @useDynLib distinct, .registration=TRUE
#' @importFrom scater sumCountsAcrossCells
#' @importFrom limma is.fullrank
#' @importFrom stats p.adjust
#' @importFrom methods is
#' @import SingleCellExperiment
#' @importFrom SummarizedExperiment assays
#' @importFrom SummarizedExperiment colData
#' @importFrom Matrix Matrix
#' @importFrom Matrix rowSums
#' @importFrom Matrix t
#' @importFrom foreach foreach
#' @importFrom foreach '%dopar%'
#' @importFrom parallel makeCluster
#' @importFrom parallel stopCluster
#' @importFrom doParallel registerDoParallel
#' @importFrom doParallel stopImplicitCluster
#' @importFrom doRNG '%dorng%'
#' @importFrom ggplot2 aes
#' @importFrom ggplot2 element_blank
#' @importFrom ggplot2 geom_hline
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 stat_ecdf
#' @importFrom ggplot2 geom_density
#' @importFrom ggplot2 stat_density
#' @importFrom ggplot2 theme
#' @importFrom ggplot2 theme_bw
#' @importFrom ggplot2 labs
NULL
