# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

perm_test <- function(P, P_2, P_3, P_4, N_breaks, cluster_ids, n_clusters, sample_ids, n_samples, group_ids_of_samples, min_non_zero_cells, counts) {
    .Call(`_distinct_perm_test`, P, P_2, P_3, P_4, N_breaks, cluster_ids, n_clusters, sample_ids, n_samples, group_ids_of_samples, min_non_zero_cells, counts)
}

perm_test_covariates <- function(P, P_2, P_3, P_4, N_breaks, cluster_ids, n_clusters, sample_ids, n_samples, group_ids_of_samples, min_non_zero_cells, counts, design) {
    .Call(`_distinct_perm_test_covariates`, P, P_2, P_3, P_4, N_breaks, cluster_ids, n_clusters, sample_ids, n_samples, group_ids_of_samples, min_non_zero_cells, counts, design)
}

perm_test_parallel_covariates <- function(P, P_2, P_3, P_4, N_breaks, sample_ids, n_samples, group_ids_of_samples, min_non_zero_cells, counts, design) {
    .Call(`_distinct_perm_test_parallel_covariates`, P, P_2, P_3, P_4, N_breaks, sample_ids, n_samples, group_ids_of_samples, min_non_zero_cells, counts, design)
}

perm_test_parallel <- function(P, P_2, P_3, P_4, N_breaks, sample_ids, n_samples, group_ids_of_samples, min_non_zero_cells, counts) {
    .Call(`_distinct_perm_test_parallel`, P, P_2, P_3, P_4, N_breaks, sample_ids, n_samples, group_ids_of_samples, min_non_zero_cells, counts)
}

